#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['lagom', 'lagom.util']

package_data = \
{'': ['*']}

setup(name='lagom',
      version='0.6.0',
      description='Lagom, a type based dependency injection container',
      author='meadsteve',
      author_email='meadsteve@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
      python_requires='~=3.7',
     )
